@@ -8,6 +8,8 @@
 #include <QMessageBox>
 #include <QInputDialog>
 #include <QUrl>
+#include <QSpinBox>
+#include <QWidgetAction>
 #include <wctype.h>
 #include <sys/stat.h>
 
@@ -32,6 +34,7 @@ static obs_hotkey_id load_first_backup_hotkey_id = OBS_INVALID_HOTKEY_ID;
 SceneCollectionManagerDialog *sceneCollectionManagerDialog = nullptr;
 
 static bool autoSaveBackup = false;
+static int autoSaveBackupMax = 30;
 static std::string customBackupDir;
 
 void ShowSceneCollectionManagerDialog()
@@ -94,6 +97,68 @@ bool GetFileSafeName(const char *name, std::string &file)
 	return true;
 }
 
+static std::string _scene_collections_path;
+
+static std::string SceneCollectionsPath()
+{
+	if (!_scene_collections_path.empty())
+		return _scene_collections_path;
+	char *path = obs_module_config_path("");
+	size_t l = strlen(path);
+	if (l == 0) {
+		bfree(path);
+		return _scene_collections_path;
+	}
+	//remove double slashes
+	char c = path[l - 1];
+	while (l > 0 && (c == '\\' || c == '/')) {
+		l--;
+		path[l] = '\0';
+		c = path[l - 1];
+	}
+	//remove module name from path
+	char *slash = strrchr(path, '/');
+	char *backslash = strrchr(path, '\\');
+	if (slash && (!backslash || slash > backslash)) {
+		*slash = '\0';
+	} else if (backslash && (!slash || backslash > slash)) {
+		*backslash = '\0';
+	}
+	//remove double slashes
+	l = strlen(path);
+	if (l > 0) {
+		c = path[l - 1];
+		while (l > 0 && (c == '\\' || c == '/')) {
+			l--;
+			path[l] = '\0';
+			c = path[l - 1];
+		}
+	}
+	//remove plugin_config from path
+	slash = strrchr(path, '/');
+	backslash = strrchr(path, '\\');
+	if (slash && (!backslash || slash > backslash)) {
+		*slash = '\0';
+	} else if (backslash && (!slash || backslash > slash)) {
+		*backslash = '\0';
+	}
+	//remove double slashes
+	l = strlen(path);
+	if (l > 0) {
+		c = path[l - 1];
+		while (l > 0 && (c == '\\' || c == '/')) {
+			l--;
+			path[l] = '\0';
+			c = path[l - 1];
+		}
+	}
+
+	_scene_collections_path = path;
+	_scene_collections_path += "/basic/scenes/";
+	bfree(path);
+	return _scene_collections_path;
+}
+
 static void BackupSceneCollection()
 {
 	const auto currentSceneCollection =
@@ -107,17 +172,17 @@ static void BackupSceneCollection()
 	if (!GetFileSafeName(currentSceneCollection, currentSafeName))
 		return;
 
-	std::string path = obs_module_config_path("../../basic/scenes/");
-
 	std::string backupName = os_generate_formatted_filename(
 		"", true, "%CCYY-%MM-%DD %hh:%mm:%ss");
-	backupName.resize(backupName.length() - 1);
+	if (backupName[backupName.size() - 1] == '.')
+		backupName.resize(backupName.length() - 1);
 
 	std::string safeName;
 	if (!GetFileSafeName(backupName.c_str(), safeName))
 		return;
 
-	std::string backupDir = path;
+	std::string filename = SceneCollectionsPath();
+	std::string backupDir = filename;
 	if (!customBackupDir.empty()) {
 		backupDir = customBackupDir;
 	}
@@ -128,7 +193,6 @@ static void BackupSceneCollection()
 	backupDir += "/";
 	os_mkdirs(backupDir.c_str());
 
-	std::string filename = path;
 	filename += currentSafeName;
 	filename += ".json";
 
@@ -138,6 +202,54 @@ static void BackupSceneCollection()
 	const auto backupFile = backupDir + safeName + ".json";
 	obs_data_save_json(data, backupFile.c_str());
 	obs_data_release(data);
+
+	if (!autoSaveBackupMax)
+		return;
+	int file_count = 0;
+	time_t time = 0;
+	do {
+
+		const auto f = backupDir + "*.json";
+		os_glob_t *glob;
+		if (os_glob(f.c_str(), 0, &glob) != 0) {
+			return;
+		}
+		std::string backupFile;
+		time = 0;
+		file_count = 0;
+		for (size_t i = 0; i < glob->gl_pathc; i++) {
+			const char *filePath = glob->gl_pathv[i].path;
+
+			if (glob->gl_pathv[i].directory)
+				continue;
+
+			const char *fn = filePath;
+			const char *slash = strrchr(filePath, '/');
+			const char *backslash = strrchr(filePath, '\\');
+			if (slash && (!backslash || backslash < slash)) {
+				fn = slash + 1;
+			} else if (backslash && (!slash || slash < backslash)) {
+				fn = backslash + 1;
+			}
+			int d, t;
+			if (sscanf(fn, "%d_%d.json", &d, &t) != 2)
+				continue;
+
+			file_count++;
+			struct stat stats {};
+			if (os_stat(filePath, &stats) == 0 &&
+			    stats.st_size > 0 &&
+			    (time == 0 || stats.st_ctime <= time)) {
+				backupFile = filePath;
+				time = stats.st_ctime;
+			}
+		}
+		os_globfree(glob);
+		if (time && file_count > autoSaveBackupMax) {
+			if (os_unlink(backupFile.c_str()) != 0)
+				return;
+		}
+	} while (time && file_count > autoSaveBackupMax);
 }
 
 void BackupSceneCollectionHotkey(void *data, obs_hotkey_id id,
@@ -192,6 +304,12 @@ bool activate_dshow_proc(void *p, obs_source_t *source)
 	if (strcmp(obs_source_get_unversioned_id(source), "dshow_input") != 0)
 		return true;
 	const bool active = *(bool *)p;
+	obs_data_t *settings = obs_source_get_settings(source);
+	if (active == obs_data_get_bool(settings, "active")) {
+		obs_data_release(settings);
+		return true;
+	}
+	obs_data_release(settings);
 	calldata_t cd = {};
 	calldata_set_bool(&cd, "active", active);
 	proc_handler_t *ph = obs_source_get_proc_handler(source);
@@ -227,8 +345,8 @@ void LoadBackupSceneCollection(const std::string sceneCollection,
 				  "scene_collection_manager_temp");
 		obs_frontend_set_current_scene_collection(
 			sceneCollection.c_str());
-		std::string path = obs_module_config_path(
-			"../../basic/scenes/scene_collection_manager_temp.json");
+		std::string path = SceneCollectionsPath() +
+				   "scene_collection_manager_temp.json";
 		os_unlink(path.c_str());
 	} else {
 		obs_frontend_set_current_scene_collection(
@@ -246,7 +364,7 @@ void LoadBackupSceneCollection(bool last)
 		config_get_string(config, "Basic", "SceneCollection");
 	const std::string filename =
 		config_get_string(config, "Basic", "SceneCollectionFile");
-	std::string path = obs_module_config_path("../../basic/scenes/");
+	std::string path = SceneCollectionsPath();
 	path += filename;
 	path += ".json";
 
@@ -342,13 +460,17 @@ static void frontend_event(obs_frontend_event event, void *)
 				  "SceneCollectionManager", "HotkeyData",
 				  data.toBase64().constData());
 		obs_data_release(save_data);
+	} else if (event == OBS_FRONTEND_EVENT_SCENE_COLLECTION_CHANGED) {
+		activate_dshow(true);
 	}
 }
 
 static void frontend_save_load(obs_data_t *, bool saving, void *)
 {
-	if (!saving && autoSaveBackup) {
-		BackupSceneCollection();
+	if (!saving) {
+		if (autoSaveBackup)
+			BackupSceneCollection();
+		activate_dshow(true);
 	}
 }
 
@@ -383,6 +505,8 @@ bool obs_module_load()
 	auto config = obs_frontend_get_global_config();
 	autoSaveBackup = config_get_bool(config, "SceneCollectionManager",
 					 "AutoSaveBackup");
+	autoSaveBackupMax = (int)config_get_int(
+		config, "SceneCollectionManager", "AutoSaveBackupMax");
 	auto *d = config_get_string(config, "SceneCollectionManager",
 				    "BackupDir");
 	if (d)
@@ -501,7 +625,7 @@ void SceneCollectionManagerDialog::on_actionImportSceneCollection_triggered()
 		"Scene Collection (*.json)");
 	if (files.isEmpty())
 		return;
-	auto path_buffer = (char *)bmalloc(MAX_PATH);
+	char path_buffer[MAX_PATH];
 	auto scene_collections_ = scene_collections;
 	for (auto file : files) {
 
@@ -545,13 +669,15 @@ void SceneCollectionManagerDialog::on_actionImportSceneCollection_triggered()
 			}
 			dir = dir.substr(0, slash + 1);
 		}
+		import_parts(data, dir.c_str());
 		try_fix_paths(data, dir.c_str(), path_buffer);
+		replace_os_specific(data);
 
-		std::string path =
-			obs_module_config_path("../../basic/scenes/");
+		std::string path = SceneCollectionsPath();
 		path += safeName;
 		path += ".json";
-		obs_data_save_json_safe(data, path.c_str(), "tmp", "bak");
+		std::string path_abs = os_get_abs_path_ptr(path.c_str());
+		obs_data_save_json_safe(data, path_abs.c_str(), "tmp", "bak");
 
 		if (replace_current) {
 			config_set_string(obs_frontend_get_global_config(),
@@ -568,7 +694,6 @@ void SceneCollectionManagerDialog::on_actionImportSceneCollection_triggered()
 		obs_data_release(data);
 		BackupSceneCollection();
 	}
-	bfree(path_buffer);
 }
 
 static bool replace(std::string &str, const char *from, const char *to)
@@ -580,6 +705,95 @@ static bool replace(std::string &str, const char *from, const char *to)
 	return true;
 }
 
+void SceneCollectionManagerDialog::import_parts(obs_data_t *data,
+						const char *dir)
+{
+	obs_data_array_t *a = obs_data_get_array(data, "imports");
+	if (!a)
+		return;
+	size_t count = obs_data_array_count(a);
+	for (size_t i = 0; i < count; i++) {
+		obs_data_t *item = obs_data_array_item(a, i);
+		if (!item)
+			continue;
+		const char *file = obs_data_get_string(item, "file");
+		obs_data_t *file_data = nullptr;
+		if (!file || !strlen(file)) {
+			obs_data_release(item);
+			continue;
+		}
+		if (os_file_exists(file)) {
+			file_data = obs_data_create_from_json_file(file);
+		}
+		if (!file_data) {
+			std::string newFile = dir;
+			newFile += file;
+			if (os_file_exists(newFile.c_str())) {
+				file_data = obs_data_create_from_json_file(
+					newFile.c_str());
+			}
+		}
+		if (!file_data) {
+			obs_data_release(item);
+			continue;
+		}
+		obs_data_item_t *item2 = obs_data_first(file_data);
+		while (item2) {
+			if (obs_data_item_gettype(item2) != OBS_DATA_ARRAY) {
+				obs_data_item_next(&item2);
+				continue;
+			}
+			obs_data_array_t *fa = obs_data_item_get_array(item2);
+			obs_data_array_t *da = obs_data_get_array(
+				data, obs_data_item_get_name(item2));
+			if (!da) {
+				da = obs_data_array_create();
+				obs_data_set_array(
+					data, obs_data_item_get_name(item2),
+					da);
+			}
+			size_t c = obs_data_array_count(fa);
+			for (size_t j = 0; j < c; j++) {
+				obs_data_t *fi = obs_data_array_item(fa, j);
+				if (!fi)
+					continue;
+				const char *name =
+					obs_data_get_string(fi, "name");
+				if (!name || !strlen(name)) {
+					obs_data_release(fi);
+					continue;
+				}
+				bool found = false;
+				size_t c2 = obs_data_array_count(da);
+				for (size_t k = 0; k < c2; k++) {
+					obs_data_t *di =
+						obs_data_array_item(da, k);
+					if (!di)
+						continue;
+					if (strcmp(obs_data_get_string(di,
+								       "name"),
+						   name) == 0) {
+						obs_data_array_erase(da, k);
+						obs_data_array_insert(da, k,
+								      fi);
+						found = true;
+						break;
+					}
+
+					obs_data_release(di);
+				}
+				if (!found) {
+					obs_data_array_push_back(da, fi);
+				}
+				obs_data_release(fi);
+			}
+			obs_data_item_next(&item2);
+		}
+		obs_data_release(file_data);
+		obs_data_release(item);
+	}
+}
+
 void SceneCollectionManagerDialog::try_fix_paths(obs_data_t *data,
 						 const char *dir,
 						 char *path_buffer)
@@ -600,7 +814,8 @@ void SceneCollectionManagerDialog::try_fix_paths(obs_data_t *data,
 				local_url = true;
 			}
 			std::size_t found = str.find_last_of("/\\");
-			if (found != std::string::npos &&
+			if (str.length() < MAX_PATH &&
+			    found != std::string::npos &&
 			    !os_file_exists(str.c_str())) {
 				while (found != std::string::npos) {
 					auto file =
@@ -693,6 +908,252 @@ void SceneCollectionManagerDialog::try_fix_paths(obs_data_t *data,
 	}
 }
 
+bool SceneCollectionManagerDialog::replace_source(obs_data_t *s, const char *id,
+						  const char *find,
+						  const char *replace, bool cs)
+{
+	if (strcmp(id, find) != 0)
+		return false;
+	obs_data_set_string(s, "id", replace);
+	const char *vid = obs_get_latest_input_type_id(replace);
+	obs_data_set_string(s, "versioned_id", vid ? vid : id);
+	if (cs) {
+		obs_data_t *c = obs_data_create();
+		obs_data_set_obj(s, "settings", c);
+		obs_data_release(c);
+	}
+
+	return true;
+}
+
+void SceneCollectionManagerDialog::make_source_mac(obs_data_t *s)
+{
+	char *id = bstrdup(obs_data_get_string(s, "id"));
+	replace_source(s, id, "game_capture", "syphon-input");
+	replace_source(s, id, "wasapi_input_capture",
+		       "coreaudio_input_capture");
+	replace_source(s, id, "wasapi_output_capture",
+		       "coreaudio_output_capture");
+	replace_source(s, id, "pulse_input_capture", "coreaudio_input_capture");
+	replace_source(s, id, "pulse_output_capture",
+		       "coreaudio_output_capture");
+	replace_source(s, id, "jack_output_capture",
+		       "coreaudio_output_capture");
+	replace_source(s, id, "alsa_input_capture", "coreaudio_input_capture");
+	replace_source(s, id, "dshow_input", "av_capture_input");
+	replace_source(s, id, "v4l2_input", "av_capture_input");
+	replace_source(s, id, "xcomposite_input", "window_capture");
+	replace_source(s, id, "xshm_input", "monitor_capture", false);
+	bfree(id);
+}
+void SceneCollectionManagerDialog::make_source_windows(obs_data_t *s)
+{
+	char *id = bstrdup(obs_data_get_string(s, "id"));
+	replace_source(s, id, "syphon-input", "game_capture");
+	replace_source(s, id, "coreaudio_input_capture",
+		       "wasapi_input_capture");
+	replace_source(s, id, "coreaudio_output_capture",
+		       "wasapi_output_capture");
+	replace_source(s, id, "pulse_input_capture", "wasapi_input_capture");
+	replace_source(s, id, "pulse_output_capture", "wasapi_output_capture");
+	replace_source(s, id, "jack_output_capture", "wasapi_output_capture");
+	replace_source(s, id, "alsa_input_capture", "wasapi_input_capture");
+	replace_source(s, id, "av_capture_input", "dshow_input");
+	replace_source(s, id, "v4l2_input", "dshow_input");
+	replace_source(s, id, "xcomposite_input", "window_capture");
+	bfree(id);
+}
+void SceneCollectionManagerDialog::make_source_linux(obs_data_t *s)
+{
+	char *id = bstrdup(obs_data_get_string(s, "id"));
+	replace_source(s, id, "coreaudio_input_capture", "pulse_input_capture");
+	replace_source(s, id, "coreaudio_output_capture",
+		       "pulse_output_capture");
+	replace_source(s, id, "wasapi_input_capture", "pulse_input_capture");
+	replace_source(s, id, "wasapi_output_capture", "pulse_output_capture");
+	replace_source(s, id, "av_capture_input", "v4l2_input");
+	replace_source(s, id, "dshow_input", "v4l2_input");
+	replace_source(s, id, "window_capture", "xcomposite_input");
+	bfree(id);
+}
+
+void SceneCollectionManagerDialog::replace_gdi_sceneitem_transform(
+	obs_data_t *item, std::map<std::string, obs_data_t *> gdi_sources)
+{
+	const char *name = obs_data_get_string(item, "name");
+	auto gdi = gdi_sources.find(name);
+	if (gdi == gdi_sources.end())
+		return;
+	struct vec2 scale;
+	obs_data_get_vec2(item, "scale", &scale);
+	scale.x *= 9.0f / 11.0f;
+	scale.y *= 9.0f / 11.0f;
+	if (obs_data_get_bool(gdi->second, "extents_wrap")) {
+		obs_data_set_int(item, "bounds_type", OBS_BOUNDS_MAX_ONLY);
+		struct vec2 bounds;
+		bounds.x = obs_data_get_double(gdi->second, "extents_cx");
+		bounds.y = obs_data_get_double(gdi->second, "extents_cy");
+		if (bounds.y < 2.0) {
+			auto font = obs_data_get_obj(gdi->second, "font");
+			auto font_size = obs_data_get_double(font, "size");
+			obs_data_release(font);
+			bounds.y = font_size;
+		}
+		obs_data_set_vec2(item, "bounds", &bounds);
+	} else {
+		obs_data_set_vec2(item, "scale", &scale);
+	}
+	const char *align_str = obs_data_get_string(gdi->second, "align");
+	const char *valign_str = obs_data_get_string(gdi->second, "valign");
+	int bounds_align = 0;
+	if (strcmp(align_str, "center") == 0) {
+		bounds_align += OBS_ALIGN_CENTER;
+		obs_data_set_int(gdi->second, "custom_width", 0);
+	} else if (strcmp(align_str, "right") == 0) {
+
+		bounds_align += OBS_ALIGN_RIGHT;
+	} else {
+
+		bounds_align += OBS_ALIGN_LEFT;
+	}
+
+	if (strcmp(valign_str, "center") == 0)
+		bounds_align += OBS_ALIGN_CENTER;
+	else if (strcmp(valign_str, "bottom") == 0)
+		bounds_align += OBS_ALIGN_BOTTOM;
+	else
+		bounds_align += OBS_ALIGN_TOP;
+	obs_data_set_int(item, "bounds_align", bounds_align);
+}
+
+void SceneCollectionManagerDialog::replace_os_specific(obs_data_t *data)
+{
+	obs_data_array_t *sources = obs_data_get_array(data, "sources");
+	if (!sources)
+		return;
+	std::map<std::string, obs_data_t *> gdi_sources;
+	auto count = obs_data_array_count(sources);
+	for (size_t i = 0; i < count; i++) {
+		obs_data_t *s = obs_data_array_item(sources, i);
+		if (!s)
+			continue;
+#ifdef WIN32
+		make_source_windows(s);
+#elif __APPLE__
+		make_source_mac(s);
+#else
+		make_source_linux(s);
+#endif
+#ifndef WIN32
+		if (strcmp(obs_data_get_string(s, "id"), "text_gdiplus") == 0) {
+			obs_data_set_string(s, "id", "text_ft2_source");
+			obs_data_set_string(s, "versioned_id",
+					    "text_ft2_source_v2");
+			obs_data_t *settings = obs_data_get_obj(s, "settings");
+			if (settings) {
+				obs_data_set_default_int(settings, "color",
+							 0xFFFFFF);
+				long long color =
+					obs_data_get_int(settings, "color");
+				color = color & 0xFFFFFF;
+				obs_data_set_default_int(settings, "opacity",
+							 100);
+				long long opacity =
+					obs_data_get_int(settings, "opacity");
+				color |= ((opacity * 255 / 100) & 0xFF) << 24;
+				obs_data_set_int(settings, "color1", color);
+				obs_data_set_int(settings, "color2", color);
+				obs_data_set_default_bool(settings,
+							  "extents_wrap", true);
+				if (obs_data_get_bool(settings,
+						      "extents_wrap")) {
+					obs_data_set_default_int(
+						settings, "extents_cx", 100);
+					obs_data_set_int(
+						settings, "custom_width",
+						obs_data_get_int(settings,
+								 "extents_cx"));
+					obs_data_set_bool(settings, "word_wrap",
+							  true);
+				}
+				gdi_sources.emplace(obs_data_get_string(s,
+									"name"),
+						    settings);
+			}
+		}
+#endif
+		obs_data_release(s);
+	}
+#ifndef WIN32
+	for (size_t i = 0; i < count; i++) {
+		obs_data_t *s = obs_data_array_item(sources, i);
+		if (!s)
+			continue;
+		if (strcmp(obs_data_get_string(s, "id"), "scene") != 0 &&
+		    strcmp(obs_data_get_string(s, "id"), "group") != 0) {
+			obs_data_release(s);
+			continue;
+		}
+
+		obs_data_t *settings = obs_data_get_obj(s, "settings");
+		obs_data_array_t *items = obs_data_get_array(settings, "items");
+		obs_data_release(settings);
+		auto item_count = obs_data_array_count(items);
+		for (size_t j = 0; j < item_count; j++) {
+			obs_data_t *item = obs_data_array_item(items, j);
+			if (!item)
+				continue;
+			replace_gdi_sceneitem_transform(item, gdi_sources);
+			obs_data_release(item);
+		}
+		obs_data_array_release(items);
+		obs_data_release(s);
+	}
+	obs_data_array_release(sources);
+
+	sources = obs_data_get_array(data, "groups");
+	count = obs_data_array_count(sources);
+	for (size_t i = 0; i < count; i++) {
+		obs_data_t *s = obs_data_array_item(sources, i);
+		if (!s)
+			continue;
+		obs_data_t *settings = obs_data_get_obj(s, "settings");
+		obs_data_array_t *items = obs_data_get_array(settings, "items");
+		obs_data_release(settings);
+		auto item_count = obs_data_array_count(items);
+		for (size_t j = 0; j < item_count; j++) {
+			obs_data_t *item = obs_data_array_item(items, j);
+			if (!item)
+				continue;
+			replace_gdi_sceneitem_transform(item, gdi_sources);
+			obs_data_release(item);
+		}
+		obs_data_array_release(items);
+		obs_data_release(s);
+	}
+	obs_data_array_release(sources);
+#endif
+	for (const auto &kv : gdi_sources) {
+		obs_data_release(kv.second);
+	}
+	auto globalAudio = {"DesktopAudioDevice1", "DesktopAudioDevice2",
+			    "AuxAudioDevice1",     "AuxAudioDevice2",
+			    "AuxAudioDevice3",     "AuxAudioDevice4"};
+	for (auto ga : globalAudio) {
+		obs_data_t *s = obs_data_get_obj(data, ga);
+		if (!s)
+			continue;
+#ifdef WIN32
+		make_source_windows(s);
+#elif __APPLE__
+		make_source_mac(s);
+#else
+		make_source_linux(s);
+#endif
+		obs_data_release(s);
+	}
+}
+
 void SceneCollectionManagerDialog::export_local_files(obs_data_t *data,
 						      std::string dir,
 						      std::string subdir)
@@ -809,8 +1270,7 @@ void SceneCollectionManagerDialog::on_actionDuplicateSceneCollection_triggered()
 		if (!GetFileSafeName(text.toUtf8().constData(), safeName))
 			return;
 
-		std::string path =
-			obs_module_config_path("../../basic/scenes/");
+		std::string path = SceneCollectionsPath();
 
 		const auto t = text.toUtf8();
 		const auto c = t.constData();
@@ -922,9 +1382,7 @@ void SceneCollectionManagerDialog::on_actionRenameSceneCollection_triggered()
 		if (!GetFileSafeName(text.toUtf8().constData(), safeName))
 			return;
 
-		std::string path =
-			obs_module_config_path("../../basic/scenes/");
-
+		std::string path = SceneCollectionsPath();
 		std::string filePath = path;
 		filePath += safeName;
 		filePath += ".json";
@@ -1018,6 +1476,16 @@ void SceneCollectionManagerDialog::on_actionAddBackup_triggered()
 		const auto filename = scene_collections.at(item->text());
 		if (!filename.length())
 			return;
+
+		const auto currentSceneCollection =
+			obs_frontend_get_current_scene_collection();
+		if (currentSceneCollection &&
+		    strlen(currentSceneCollection) > 0 &&
+		    item->text() == QString::fromUtf8(currentSceneCollection)) {
+			obs_frontend_save();
+		}
+		bfree(currentSceneCollection);
+
 		bool ok;
 		std::string defaultName = os_generate_formatted_filename(
 			"", true, "%CCYY-%MM-%DD %hh:%mm:%ss");
@@ -1102,6 +1570,7 @@ void SceneCollectionManagerDialog::on_actionConfigBackup_triggered()
 	auto a = m.addAction(QString::fromUtf8(obs_module_text("Rename")));
 	connect(a, SIGNAL(triggered()), this,
 		SLOT(on_actionRenameBackup_triggered()));
+	m.addSeparator();
 
 	a = m.addAction(QString::fromUtf8(obs_module_text("AutoBackup")));
 	a->setCheckable(true);
@@ -1113,16 +1582,43 @@ void SceneCollectionManagerDialog::on_actionConfigBackup_triggered()
 				"AutoSaveBackup", autoSaveBackup);
 	});
 
+	QWidget *maxRow = new QWidget(&m);
+	auto hl = new QHBoxLayout;
+	maxRow->setLayout(hl);
+
+	QSpinBox *maxSpin = new QSpinBox(&m);
+	maxSpin->setMinimum(0);
+	maxSpin->setMaximum(1000);
+	maxSpin->setSingleStep(1);
+	maxSpin->setValue(autoSaveBackupMax);
+
+	hl->addWidget(maxSpin);
+
+	QWidgetAction *maxAction = new QWidgetAction(&m);
+	maxAction->setDefaultWidget(maxRow);
+
+	connect(maxSpin, (void(QSpinBox::*)(int)) & QSpinBox::valueChanged,
+		[](int val) {
+			autoSaveBackupMax = val;
+			const auto config = obs_frontend_get_global_config();
+			config_set_int(config, "SceneCollectionManager",
+				       "AutoSaveBackupMax", autoSaveBackupMax);
+		});
+
+	m.addMenu(QString::fromUtf8(obs_module_text("Max")))
+		->addAction(maxAction);
+
+	m.addSeparator();
+
 	auto dirMenu =
 		m.addMenu(QString::fromUtf8(obs_module_text("BackupDir")));
 	a = dirMenu->addAction(QString::fromUtf8(obs_module_text("ShowDir")));
 	connect(a, &QAction::triggered, [] {
 		QUrl url;
 		if (customBackupDir.empty()) {
-			auto ptr = os_get_abs_path_ptr(
-				obs_module_config_path("../../basic/scenes/"));
-			url = QUrl::fromLocalFile(QString::fromUtf8(ptr));
-			bfree(ptr);
+			auto ptr = SceneCollectionsPath();
+			url = QUrl::fromLocalFile(
+				QString::fromUtf8(ptr.c_str()));
 		} else {
 			url = QUrl::fromLocalFile(
 				QString::fromUtf8(customBackupDir.c_str()));
@@ -1302,10 +1798,15 @@ void SceneCollectionManagerDialog::on_backupList_itemDoubleClicked(
 
 void SceneCollectionManagerDialog::ReadSceneCollections()
 {
-	std::string path = obs_module_config_path("../../basic/scenes/*.json");
+	std::string path = SceneCollectionsPath() + "*.json";
+	if (path.empty()) {
+		blog(LOG_WARNING, "Failed to get scene collections path");
+		return;
+	}
 	os_glob_t *glob;
 	if (os_glob(path.c_str(), 0, &glob) != 0) {
-		blog(LOG_WARNING, "Failed to glob scene collections");
+		blog(LOG_WARNING, "Failed to glob scene collections in:%s",
+		     path.c_str());
 		return;
 	}
 	scene_collections.clear();
@@ -1318,14 +1819,23 @@ void SceneCollectionManagerDialog::ReadSceneCollections()
 
 		auto *data =
 			obs_data_create_from_json_file_safe(filePath, "bak");
+		if (!data)
+			continue;
 		std::string name = obs_data_get_string(data, "name");
 		obs_data_release(data);
 
 		/* if no name found, use the file name as the name
 		 * (this only happens when switching to the new version) */
 		if (name.empty()) {
-			name = strrchr(filePath, '/') + 1;
-			name.resize(name.size() - 5);
+			auto p = strrchr(filePath, '/');
+			if (!p)
+				p = strrchr(filePath, '\\');
+			if (p)
+				name = p + 1;
+			else
+				name = filePath;
+			if (name.size() > 5)
+				name.resize(name.size() - 5);
 		}
 		scene_collections[QString::fromUtf8(name.c_str())] = filePath;
 	}
@@ -1333,7 +1843,8 @@ void SceneCollectionManagerDialog::ReadSceneCollections()
 }
 
 SceneCollectionManagerDialog::SceneCollectionManagerDialog(QMainWindow *parent)
-	: QDialog(parent), ui(new Ui::SceneCollectionManagerDialog)
+	: QDialog(parent),
+	  ui(new Ui::SceneCollectionManagerDialog)
 {
 	ui->setupUi(this);
 
